import { __decorate, __metadata } from "tslib";
import { inject, injectable } from 'mana-syringe';
import { EntityManager } from './EntityManager';
var counter = 0;
/**
 * A bag of components
 * @see https://github.com/mzaks/EntitasCookBook/blob/master/chapters/1_ingredients/102_entity.md
 */

var Entity =
/** @class */
function () {
  function Entity() {
    /**
     * A unique ID for this entity.
     */
    this.id = counter++;
    this.components = {};
  }

  Entity.prototype.getId = function () {
    return this.id;
  };

  Entity.prototype.getComponents = function () {
    return this.components;
  };

  Entity.prototype.setName = function (name) {
    this.name = name;
  };

  Entity.prototype.getName = function () {
    return this.name || "" + this.id;
  };

  Entity.prototype.getComponent = function (clazz) {
    return this.components[clazz.tag];
  };

  Entity.prototype.hasComponent = function (clazz) {
    return !!~Object.keys(this.components).indexOf(clazz.tag);
  };

  Entity.prototype.hasAllComponents = function (clazzes) {
    var _this = this;

    return clazzes.every(function (clazz) {
      return _this.hasComponent(clazz);
    });
  };

  Entity.prototype.hasAnyComponents = function (clazzes) {
    var _this = this;

    return clazzes.some(function (clazz) {
      return _this.hasComponent(clazz);
    });
  };

  Entity.prototype.addComponent = function (clazz, values) {
    return this.entityManager.addComponentToEntity(this, clazz, values);
  };

  Entity.prototype.removeComponent = function (clazz, forceImmediate) {
    if (forceImmediate === void 0) {
      forceImmediate = false;
    }

    this.entityManager.removeComponentFromEntity(this, clazz, forceImmediate);
    return this;
  }; // public removeAllComponents() {
  //   return this;
  // }


  Entity.prototype.reset = function () {//
  };

  Entity.prototype.destroy = function () {
    // TODO: remove components first
    // for (const name in this.components) {
    //   this.entityManager.removeComponentFromEntity(this, this.components[name]);
    // }
    this.entityManager.destroyEntity(this);
  };

  Entity.prototype.cast = function (component, componentClass) {
    return !!(component && component instanceof componentClass);
  };

  __decorate([inject(EntityManager), __metadata("design:type", EntityManager)], Entity.prototype, "entityManager", void 0);

  Entity = __decorate([injectable()], Entity);
  return Entity;
}();

export { Entity };