import { __decorate } from "tslib";
import { injectable } from 'mana-syringe';

var ObjectPool =
/** @class */
function () {
  function ObjectPool() {
    this.count = 0;
    this.freeList = [];
  }

  ObjectPool.prototype.init = function (T, initialSize) {
    this.objectFactory = T;

    if (typeof initialSize !== 'undefined') {
      this.expand(initialSize);
    }
  };

  ObjectPool.prototype.acquire = function () {
    if (this.freeList.length <= 0) {
      this.expand(Math.round(this.count * 0.2) + 1);
    }

    return this.freeList.pop();
  };

  ObjectPool.prototype.release = function (item) {
    item.reset();
    this.freeList.push(item);
  };

  ObjectPool.prototype.expand = function (count) {
    for (var n = 0; n < count; n++) {
      var clone = this.objectFactory();
      this.freeList.push(clone);
    }

    this.count += count;
  };

  ObjectPool = __decorate([injectable()], ObjectPool);
  return ObjectPool;
}();

export { ObjectPool };