import { Component, ComponentConstructor } from './Component';
import { Entity } from './Entity';
import { System, SystemConstructor } from './System';
interface IWorldLifecycle {
    execute(): void;
    stop(): void;
    resume(): void;
}
/**
 * the context of ECS
 * @see https://github.com/sschmid/Entitas-CSharp/blob/master/Entitas/Entitas/Context/IContext.cs
 * @example
 * ```
 * // register components
 * world
    .registerComponent(C1)
    .registerComponent(C2)
    .registerComponent(C3);
  
  // register systems
 * ```
 */
export declare class World implements IWorldLifecycle {
    private entityPoolFactory;
    private entityManager;
    private componentManager;
    private systemManager;
    private lastMillis;
    private enabled;
    createEntity(name?: string): Entity;
    getEntityByName(name: string): Entity;
    registerComponent<C extends Component>(clazz: ComponentConstructor<C>): this;
    registerSystem<S extends System>(clazz: SystemConstructor<S>): this;
    execute(delta?: number, millis?: number): void;
    stop(): void;
    resume(): void;
    destroy(): void;
}
export {};
