import { Module, injectable, decorate } from 'mana-syringe';
import { EventEmitter } from 'eventemitter3';
import { ObjectPool } from './ObjectPool';
import { Component } from './Component';
import { IDENTIFIER } from './identifier';
import { EntityManager } from './EntityManager';
import { Entity } from './Entity';
import { World } from './World';
import { ComponentManager } from './ComponentManager';
import { SystemManager } from './SystemManager';
import { Matcher } from './Matcher';
var containerModule = Module(function (register) {
  decorate(injectable(), EventEmitter);
  register({
    token: IDENTIFIER.ComponentRegistry,
    useFactory: function useFactory(context) {
      return function (clazz) {
        // if (!context.container.isBound(clazz)) {
        context.container.register(clazz); // }
      };
    }
  });
  register({
    token: IDENTIFIER.SystemRegistry,
    useDynamic: function useDynamic(context) {
      return {
        register: function register(clazz) {
          context.container.register(clazz);
        },
        get: function get(clazz) {
          return context.container.get(clazz);
        },
        has: function has(clazz) {
          return context.container.isBound(clazz);
        }
      };
    }
  });
  register({
    token: IDENTIFIER.ComponentPoolFactory,
    useFactory: function useFactory(context) {
      var factoryRegistry = {};
      return function (clazz) {
        var componentPool = factoryRegistry[clazz.tag];

        if (!componentPool) {
          componentPool = context.container.get(ObjectPool);
          componentPool.init(function () {
            var isBound = context.container.isBound(clazz);

            if (!isBound) {
              throw new Error("Component \"" + clazz.tag + "\" is not registered, please call registerComponent() first.");
            }

            var component = context.container.get(clazz);
            return component;
          }, 10);
          factoryRegistry[clazz.tag] = componentPool;
        }

        return componentPool;
      };
    }
  });
  register({
    token: IDENTIFIER.EntityPoolFactory,
    useFactory: function useFactory(context) {
      var entityPool;
      return function () {
        if (!entityPool) {
          entityPool = context.container.get(ObjectPool);
          entityPool.init(function () {
            return context.container.get(Entity);
          }, 10);
        }

        return entityPool;
      };
    }
  });
  register(EntityManager);
  register(ComponentManager);
  register(SystemManager);
  register(Entity);
  register(ObjectPool);
  register(World);
});
export * from './System';
export { Matcher, Entity, Component, World, EntityManager, containerModule };