import { ComponentConstructor, Component } from './Component';
import { ILifecycle } from './ObjectPool';
/**
 * A bag of components
 * @see https://github.com/mzaks/EntitasCookBook/blob/master/chapters/1_ingredients/102_entity.md
 */
export declare class Entity implements ILifecycle {
    /**
     * A unique ID for this entity.
     */
    protected id: number;
    protected name: string;
    protected components: Record<string, Component>;
    private entityManager;
    getId(): number;
    getComponents(): Record<string, Component>;
    setName(name: string): void;
    getName(): string;
    getComponent<C extends Component>(clazz: ComponentConstructor<C>): C;
    hasComponent<C extends Component>(clazz: ComponentConstructor<C>): boolean;
    hasAllComponents<C extends Component>(clazzes: ComponentConstructor<C>[]): boolean;
    hasAnyComponents<C extends Component>(clazzes: ComponentConstructor<C>[]): boolean;
    addComponent<C extends Component>(clazz: ComponentConstructor<C>, values?: Partial<Omit<C, keyof Component>>): C;
    removeComponent<C extends Component>(clazz: ComponentConstructor<C>, forceImmediate?: boolean): this;
    reset(): void;
    destroy(): void;
    cast<C extends Component>(component: Component | undefined | null, componentClass: ComponentConstructor<C>): component is C;
}
