import { Component, ComponentConstructor } from './Component';
import type { Entity } from './Entity';
interface IMatcher {
    matches(entity: Entity): boolean;
}
interface ICompoundMatcher<C extends Component> extends IMatcher {
    allOfComponentCtors: ComponentConstructor<C>[];
    anyOfComponentCtors: ComponentConstructor<C>[];
    noneOfComponentCtors: ComponentConstructor<C>[];
}
interface INoneOfMatcher<C extends Component> extends ICompoundMatcher<C> {
}
interface IAnyOfMatcher<C extends Component> extends ICompoundMatcher<C> {
    noneOf(...clazzes: ComponentConstructor<C>[]): INoneOfMatcher<C>;
}
interface IAllOfMatcher<C extends Component> extends ICompoundMatcher<C> {
    anyOf(...clazzes: ComponentConstructor<C>[]): IAnyOfMatcher<C>;
    noneOf(...clazzes: ComponentConstructor<C>[]): INoneOfMatcher<C>;
}
/**
 * Query language of ECS
 * @see https://github.com/mzaks/EntitasCookBook/blob/master/chapters/1_ingredients/104_group.md#matcher
 * @example
 * ```
 * matcher.allOf(Position, Velocity).noneOf(NotMovable)
 * ```
 */
export declare class Matcher<C extends Component = Component> implements IAllOfMatcher<C>, IAnyOfMatcher<C>, INoneOfMatcher<C> {
    allOfComponentCtors: ComponentConstructor<C>[];
    anyOfComponentCtors: ComponentConstructor<C>[];
    noneOfComponentCtors: ComponentConstructor<C>[];
    allOf(...clazzes: ComponentConstructor<C>[]): Matcher<C>;
    anyOf(...clazzes: ComponentConstructor<C>[]): Matcher<C>;
    noneOf(...clazzes: ComponentConstructor<C>[]): Matcher<C>;
    matches(entity: Entity): boolean;
    hash(): string;
}
export {};
