"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Matcher = void 0;

var _tslib = require("tslib");

/**
 * Query language of ECS
 * @see https://github.com/mzaks/EntitasCookBook/blob/master/chapters/1_ingredients/104_group.md#matcher
 * @example
 * ```
 * matcher.allOf(Position, Velocity).noneOf(NotMovable)
 * ```
 */
var Matcher =
/** @class */
function () {
  function Matcher() {
    this.allOfComponentCtors = [];
    this.anyOfComponentCtors = [];
    this.noneOfComponentCtors = [];
  }

  Matcher.prototype.allOf = function () {
    var clazzes = [];

    for (var _i = 0; _i < arguments.length; _i++) {
      clazzes[_i] = arguments[_i];
    }

    this.allOfComponentCtors = clazzes;
    return this;
  };

  Matcher.prototype.anyOf = function () {
    var clazzes = [];

    for (var _i = 0; _i < arguments.length; _i++) {
      clazzes[_i] = arguments[_i];
    }

    this.anyOfComponentCtors = clazzes;
    return this;
  };

  Matcher.prototype.noneOf = function () {
    var clazzes = [];

    for (var _i = 0; _i < arguments.length; _i++) {
      clazzes[_i] = arguments[_i];
    }

    this.noneOfComponentCtors = clazzes;
    return this;
  };

  Matcher.prototype.matches = function (entity) {
    var matchesAllOf = !this.allOfComponentCtors.length ? true : entity.hasAllComponents(this.allOfComponentCtors);
    var matchesAnyOf = !this.anyOfComponentCtors.length ? true : entity.hasAnyComponents(this.anyOfComponentCtors);
    var matchesNoneOf = !this.noneOfComponentCtors.length ? true : !entity.hasAnyComponents(this.noneOfComponentCtors);
    return matchesAllOf && matchesAnyOf && matchesNoneOf;
  };

  Matcher.prototype.hash = function () {
    return (0, _tslib.__spreadArray)((0, _tslib.__spreadArray)((0, _tslib.__spreadArray)([], (0, _tslib.__read)(this.allOfComponentCtors), false), (0, _tslib.__read)(this.anyOfComponentCtors), false), (0, _tslib.__read)(this.noneOfComponentCtors), false).map(function (c) {
      return c.tag;
    }).join();
  };

  return Matcher;
}();

exports.Matcher = Matcher;