"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SystemManager = void 0;

var _tslib = require("tslib");

var _manaSyringe = require("mana-syringe");

var _EntityManager = require("./EntityManager");

var _identifier = require("./identifier");

var SystemManager =
/** @class */
function () {
  function SystemManager() {
    this.systems = [];
  }

  SystemManager.prototype.registerSystem = function (clazz) {
    if (!this.registry.has(clazz)) {
      this.registry.register(clazz);
      var system_1 = this.registry.get(clazz);
      var systemCtor_1 = system_1.constructor;

      if (system_1.initialize) {
        // TODO: support async init @see https://github.com/ecsyjs/ecsy/issues/20
        system_1.initialize();
      } else {
        system_1.initialized = true;
      }

      this.entityManager.on(_EntityManager.COMPONENT_EVENT.Added, function (entity) {
        if (system_1.onEntityAdded && systemCtor_1.trigger && systemCtor_1.trigger.matches(entity)) {
          system_1.onEntityAdded(entity);
        }
      });
      this.entityManager.on(_EntityManager.COMPONENT_EVENT.Remove, function (entity) {
        if (system_1.onEntityRemoved && systemCtor_1.trigger && systemCtor_1.trigger.matches(entity)) {
          system_1.onEntityRemoved(entity);
        }
      });
      this.systems.push(system_1);
      this.systems.sort(function (a, b) {
        return (a.constructor.priority || 0) - (b.constructor.priority || 0);
      });
    }
  };

  SystemManager.prototype.execute = function (delta, millis) {
    var e_1, _a;

    try {
      for (var _b = (0, _tslib.__values)(this.systems), _c = _b.next(); !_c.done; _c = _b.next()) {
        var system = _c.value;

        if (system.initialized && system.execute) {
          // const t1 = performance.now();
          system.execute(this.getEntities(system), delta, millis); // console.log((system.constructor as SystemConstructor<System>).tag, performance.now() - t1);
        }
      }
    } catch (e_1_1) {
      e_1 = {
        error: e_1_1
      };
    } finally {
      try {
        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
      } finally {
        if (e_1) throw e_1.error;
      }
    }
  };

  SystemManager.prototype.destroy = function () {
    var _this = this;

    this.systems.forEach(function (system) {
      if (system.initialized && system.tearDown) {
        system.tearDown(_this.getEntities(system));
      }
    });
  };

  SystemManager.prototype.getEntities = function (system) {
    var systemCtor = system.constructor;
    return systemCtor.trigger ? this.entityManager.queryByMatcher(systemCtor.trigger) : this.entityManager.getAllEntities();
  };

  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_identifier.IDENTIFIER.SystemRegistry), (0, _tslib.__metadata)("design:type", Object)], SystemManager.prototype, "registry", void 0);
  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_EntityManager.EntityManager), (0, _tslib.__metadata)("design:type", _EntityManager.EntityManager)], SystemManager.prototype, "entityManager", void 0);
  SystemManager = (0, _tslib.__decorate)([(0, _manaSyringe.singleton)()], SystemManager);
  return SystemManager;
}();

exports.SystemManager = SystemManager;