"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.World = void 0;

var _tslib = require("tslib");

var _manaSyringe = require("mana-syringe");

var _ComponentManager = require("./ComponentManager");

var _EntityManager = require("./EntityManager");

var _identifier = require("./identifier");

var _SystemManager = require("./SystemManager");

/**
 * the context of ECS
 * @see https://github.com/sschmid/Entitas-CSharp/blob/master/Entitas/Entitas/Context/IContext.cs
 * @example
 * ```
 * // register components
 * world
    .registerComponent(C1)
    .registerComponent(C2)
    .registerComponent(C3);
  
  // register systems
 * ```
 */
var World =
/** @class */
function () {
  function World() {
    this.lastMillis = new Date().getTime();
    this.enabled = true;
  }

  World.prototype.createEntity = function (name) {
    if (name === void 0) {
      name = '';
    }

    var entity = this.entityPoolFactory().acquire();
    entity.setName(name);
    return this.entityManager.createEntity(entity);
  };

  World.prototype.getEntityByName = function (name) {
    return this.entityManager.getEntityByName(name);
  };

  World.prototype.registerComponent = function (clazz) {
    this.componentManager.registerComponent(clazz);
    return this;
  };

  World.prototype.registerSystem = function (clazz) {
    this.systemManager.registerSystem(clazz);
    return this;
  };

  World.prototype.execute = function (delta, millis) {
    if (!delta) {
      millis = new Date().getTime();
      delta = millis - this.lastMillis;
      this.lastMillis = millis;
    }

    if (this.enabled) {
      this.systemManager.execute(delta, millis);
    }
  };

  World.prototype.stop = function () {
    this.enabled = false;
  };

  World.prototype.resume = function () {
    this.enabled = true;
  };

  World.prototype.destroy = function () {
    this.systemManager.destroy();
    this.entityManager.destroy(); // this.componentManager.destroy();
  };

  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_identifier.IDENTIFIER.EntityPoolFactory), (0, _tslib.__metadata)("design:type", Function)], World.prototype, "entityPoolFactory", void 0);
  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_EntityManager.EntityManager), (0, _tslib.__metadata)("design:type", _EntityManager.EntityManager)], World.prototype, "entityManager", void 0);
  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_ComponentManager.ComponentManager), (0, _tslib.__metadata)("design:type", _ComponentManager.ComponentManager)], World.prototype, "componentManager", void 0);
  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_SystemManager.SystemManager), (0, _tslib.__metadata)("design:type", _SystemManager.SystemManager)], World.prototype, "systemManager", void 0);
  World = (0, _tslib.__decorate)([(0, _manaSyringe.singleton)()], World);
  return World;
}();

exports.World = World;