"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  containerModule: true,
  Component: true,
  EntityManager: true,
  Entity: true,
  World: true,
  Matcher: true
};
Object.defineProperty(exports, "Component", {
  enumerable: true,
  get: function get() {
    return _Component.Component;
  }
});
Object.defineProperty(exports, "Entity", {
  enumerable: true,
  get: function get() {
    return _Entity.Entity;
  }
});
Object.defineProperty(exports, "EntityManager", {
  enumerable: true,
  get: function get() {
    return _EntityManager.EntityManager;
  }
});
Object.defineProperty(exports, "Matcher", {
  enumerable: true,
  get: function get() {
    return _Matcher.Matcher;
  }
});
Object.defineProperty(exports, "World", {
  enumerable: true,
  get: function get() {
    return _World.World;
  }
});
exports.containerModule = void 0;

var _manaSyringe = require("mana-syringe");

var _eventemitter = require("eventemitter3");

var _ObjectPool = require("./ObjectPool");

var _Component = require("./Component");

var _identifier = require("./identifier");

var _EntityManager = require("./EntityManager");

var _Entity = require("./Entity");

var _World = require("./World");

var _ComponentManager = require("./ComponentManager");

var _SystemManager = require("./SystemManager");

var _Matcher = require("./Matcher");

var _System = require("./System");

Object.keys(_System).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _System[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function get() {
      return _System[key];
    }
  });
});
var containerModule = (0, _manaSyringe.Module)(function (register) {
  (0, _manaSyringe.decorate)((0, _manaSyringe.injectable)(), _eventemitter.EventEmitter);
  register({
    token: _identifier.IDENTIFIER.ComponentRegistry,
    useFactory: function useFactory(context) {
      return function (clazz) {
        // if (!context.container.isBound(clazz)) {
        context.container.register(clazz); // }
      };
    }
  });
  register({
    token: _identifier.IDENTIFIER.SystemRegistry,
    useDynamic: function useDynamic(context) {
      return {
        register: function register(clazz) {
          context.container.register(clazz);
        },
        get: function get(clazz) {
          return context.container.get(clazz);
        },
        has: function has(clazz) {
          return context.container.isBound(clazz);
        }
      };
    }
  });
  register({
    token: _identifier.IDENTIFIER.ComponentPoolFactory,
    useFactory: function useFactory(context) {
      var factoryRegistry = {};
      return function (clazz) {
        var componentPool = factoryRegistry[clazz.tag];

        if (!componentPool) {
          componentPool = context.container.get(_ObjectPool.ObjectPool);
          componentPool.init(function () {
            var isBound = context.container.isBound(clazz);

            if (!isBound) {
              throw new Error("Component \"" + clazz.tag + "\" is not registered, please call registerComponent() first.");
            }

            var component = context.container.get(clazz);
            return component;
          }, 10);
          factoryRegistry[clazz.tag] = componentPool;
        }

        return componentPool;
      };
    }
  });
  register({
    token: _identifier.IDENTIFIER.EntityPoolFactory,
    useFactory: function useFactory(context) {
      var entityPool;
      return function () {
        if (!entityPool) {
          entityPool = context.container.get(_ObjectPool.ObjectPool);
          entityPool.init(function () {
            return context.container.get(_Entity.Entity);
          }, 10);
        }

        return entityPool;
      };
    }
  });
  register(_EntityManager.EntityManager);
  register(_ComponentManager.ComponentManager);
  register(_SystemManager.SystemManager);
  register(_Entity.Entity);
  register(_ObjectPool.ObjectPool);
  register(_World.World);
});
exports.containerModule = containerModule;